-- to access configuration control chip, use BAR1
-- BAR1 occupies 16 byte space
-- 0x0 write address
-- 0x4 write/read data
-- 0x8 resets virtex2 chip
-- 0xC write reconfigure commmand
-- All accesses have to be 32-bit word. Byte enable signals are assumed to be all active.
-- flash memory address register has address 0x0 and is write-only. Only data bits 20-0 are used
-- flash mmeory data register has address 0x4. write access uses bits 7-0 for flash data.
-- read access returns
-- bits 31-14 all '0'
-- bit 13 is '1' indicates that configuration started from 0xc0000 
-- bit 12 pci3 done signal  
-- bit 11 pci2 done signal  
-- bit 10 pci1 done signal  
-- bit 9 virtex done signal  
-- bit 8 virtex init signal
-- bit 7-0 are flash data
-- address register does not auto-increment
-- write to data register automatically starts the execution of the command
-- for flash memory operation, please refer to AM29F080B data sheet
--
-- Following any wirte/read operation, flash controller automatically reads the flash for later access
-- It is important to notice that any read access always gets the data resulted from the last read/write access.
--  
-- Any access takes 2+ microsecond
-- to finish. Any new access before it is ready again will result in a retry. 
--
-- to start a reconfiguration, first write the starting address to flash memory address register
-- for normal configuration, set address to 0xCF100, it reconfigures all FPGAs
-- for pci3 interface backup configuration, set address to BF100, it only reconfigures pci3 interface
-- then write 0xa5 to data register at address 0xC, this starts the reconfiguration
--
-- pci3 interface has two sets of configuration data in the flash memory, one starts at 0xc0000, and
-- the other starts at 0xf0000. Never update two sets at the same time. Always reconfigure after updating
-- one set. Only if it works correctly, then start updating the other.
-- flash memory contents
-- 0xc0000 - pci3 backup
-- 0xd0000 - pci1
-- 0xe0000 - pci2
-- 0xf0000 - pci3 normal
-- 0x100000 - virtex2
--
-- dcc interface, see dccv4.txt
-- dcc_ctl_o(0) = '0' valid data
-- dcc_ctl_o(1) = toggles DDR ram read cache miss, generate retry
-- dcc_ctl_o(2) not used
-- dcc_ctl_o(3) xilinx done signal
-- dcc_ctl_i(1 downto 0) = "00"	NOP
-- dcc_ctl_i(1 downto 0) = "01"	read data
-- dcc_ctl_i(1 downto 0) = "10"	write address
-- dcc_ctl_i(1 downto 0) = "11"	write data
-- dcc_ctl_i(2) not used 
-- dcc_ctl_i(3) = '0'	resets virtex DCM
LIBRARY ieee;
USE ieee.std_logic_1164.all;
USE ieee.std_logic_unsigned.all;
USE ieee.std_logic_arith.all;

ENTITY pci3_if IS

	PORT
	(
-- pci megafunction interface
-- inputs        
               clock:		in std_logic;  -- PCI clock input, rising edge active
               rstn:		in std_logic;  --- PCI reset input 
               lt_framen:	in std_logic;  --- PCI interface needs local access
               lt_ackn:		in std_logic;  --- PCI inetrface is ready for r/w data
               lt_dxfrn:	in std_logic;  --- last target data transfer successful
               lt_tsr:		in std_logic_vector(11 downto 0);  --- target transaction status
               l_dato:		in std_logic_vector(31 downto 0);  --- local data input
               l_adro:		in std_logic_vector(31 downto 0);  --- local address input
               l_cmdo:		in std_logic_vector(3 downto 0);  --- local command input
-- outputs
               l_cbeni:		out std_logic_vector(3 downto 0);  --- local command input
               lt_rdyn:		out std_logic;  --- local target ready for r/w data
               lt_discn:	out std_logic;  --- local target disconnect output
               lt_abortn:	out std_logic;  --- local target abort output
	           l_adi:		out std_logic_vector(31 downto 0);  --- local data output
-- dcc interface    
    	       dcc_data:	inout std_logic_vector(15 downto 0);
    		   dcc_ctl_o:	in std_logic_vector(5 downto 0);
    		   dcc_ctl_i:	out std_logic_vector(5 downto 0);
		       dcc_clk:		in std_logic;
-- configuration controls
-- inputs
		      pci_sp_i:	in	std_logic_vector(3 downto 0);
-- outputs
		      pci_sp_o:	out	std_logic_vector(8 downto 0)
    );
	
END pci3_if;
ARCHITECTURE my_arch OF pci3_if IS
COMPONENT lpm_ram_dp
   GENERIC ( LPM_WIDTH: POSITIVE :=32;
             LPM_WIDTHAD: POSITIVE := 4;
             LPM_NUMWORDS: NATURAL := 16;
             LPM_TYPE: STRING := "LPM_RAM_DP";
             LPM_INDATA: STRING := "REGISTERED";
             LPM_OUTDATA: STRING := "UNREGISTERED";
             LPM_RDADDRESS_CONTROL: STRING := "REGISTERED";
             LPM_WRADDRESS_CONTROL: STRING := "REGISTERED";
             LPM_FILE: STRING := "UNUSED";
             LPM_HINT: STRING := "UNUSED"

           );
   PORT    ( rdaddress, wraddress: IN STD_LOGIC_VECTOR(LPM_WIDTHAD-1 DOWNTO 0);
             rdclock, wrclock: IN STD_LOGIC := '0';
             rden, rdclken, wrclken: IN STD_LOGIC := '1';
             wren: IN STD_LOGIC; 
             data: IN STD_LOGIC_VECTOR(LPM_WIDTH-1 DOWNTO 0);
             q: OUT STD_LOGIC_VECTOR(LPM_WIDTH-1 DOWNTO 0));
END COMPONENT;
COMPONENT lpm_counter
   GENERIC (LPM_WIDTH: POSITIVE := 14;
      LPM_MODULUS: NATURAL := 0;
      LPM_DIRECTION: STRING := "UNUSED";
      LPM_AVALUE: STRING := "UNUSED";
      LPM_SVALUE: STRING := "UNUSED";
      LPM_PVALUE: STRING := "UNUSED";
      LPM_TYPE: STRING := "LPM_COUNTER";
      LPM_HINT: STRING := "UNUSED");
   PORT (data: IN STD_LOGIC_VECTOR(LPM_WIDTH-1 DOWNTO 0) := (OTHERS => '0');
      clock: IN STD_LOGIC;
      cin: IN STD_LOGIC := '0';
      clk_en: IN STD_LOGIC := '1';
      cnt_en: IN STD_LOGIC := '1';
      updown: IN STD_LOGIC := '1';
      sload: IN STD_LOGIC := '0';
      sset: IN STD_LOGIC := '0';
      sclr: IN STD_LOGIC := '0';
      aload: IN STD_LOGIC := '0';
      aset: IN STD_LOGIC := '0';
      aclr: IN STD_LOGIC := '0';
      cout: OUT STD_LOGIC := '0';
      q: OUT STD_LOGIC_VECTOR(LPM_WIDTH-1 DOWNTO 0));
END COMPONENT;
component gray
  generic( n : positive := 4);
  port (clk: in std_logic;
         ec: in std_logic;
        clrn: in std_logic;
         q: out std_logic_vector(n-1 downto 0));
end component;
-- configuration interface
constant tc: integer := 31;
signal conf_tmp0: std_logic_vector(19 downto 0);
signal conf_tmp1: std_logic_vector(23 downto 0);
signal conf_data, conf_tgl, conf_write, conf_din, ver_din, ver_din_q: std_logic;
signal conf_clk: std_logic;
-- state machine for configuration interface
signal bar1_read, discon: std_logic;
-- dcc interface signals
signal rfifo_dout : std_logic_vector(31 downto 0);
signal dav,high_word,wfifo_we,wfifo_re: std_logic;
signal rfifo_re,is_high,rfifo_weh,rfifo_wel,rfifo_empty,flush_rfifo: std_logic;
signal ready1: std_logic;
signal wfifo_wa, wfifo_ra, sync_wwa, sync2_wwa: std_logic_vector(3 downto 0);
signal wfifo_din, wfifo_dout : std_logic_vector(34 downto 0);
signal dcc_ctl_oq: std_logic_vector(5 downto 0);
signal dcc_data_i,dcc_data_o,t_dcc_data: std_logic_vector(15 downto 0);
signal rfifo_ra, rfifo_wa : std_logic_vector(4 downto 0);
signal clr_rfifo_wa_n : std_logic;
signal wr_ad_sync,wr_ad_syncp: std_logic_vector(4 downto 0);
signal rst_dccn,dcc_ready: std_logic;
signal dcc_init: unsigned(20 downto 0);
signal ld_addr, dcc_rd_rdy: std_logic;
signal retry: std_logic_vector(1 downto 0);
signal select_ram: std_logic;
signal pci_cycle: std_logic;
signal start: std_logic;
signal shift_sync: std_logic_vector(2 downto 0);
signal input_sel: std_logic;
signal start_sync: std_logic;
signal out_shift: std_logic;
signal conf_dout: std_logic;
signal conf_sel: std_logic;
signal conf_tgl_sync: std_logic_vector(2 downto 0);
signal en_count: std_logic;
signal en_count_q: std_logic;
signal en_count_dl: std_logic;
signal ec_count: std_logic;
signal en_count_sync: std_logic_vector(2 downto 0);
signal count: unsigned(4 downto 0);
signal load_addr: std_logic_vector(1 downto 0);
signal read_flash: std_logic;
signal write_flash: std_logic_vector(1 downto 0);
signal shift_flag: std_logic_vector(1 downto 0);
signal shift_flag_p: std_logic;
signal shift_ver_in: std_logic_vector(1 downto 0);
signal shift_data_in: std_logic_vector(1 downto 0);
signal shift_data_out: std_logic_vector(1 downto 0);
signal rd_addr: std_logic_vector(4 downto 0);
BEGIN
dcc_ctl_i(5 downto 4) <= "00";
-- instantiate write fifo's dpram, which is mainly used for synchronization purpose only. It is assumed that the
-- dcc side has double the clock rate so it will never get full.
wfifo: lpm_ram_dp
   generic map(LPM_WIDTH => 35,LPM_OUTDATA => "REGISTERED", LPM_RDADDRESS_CONTROL => "UNREGISTERED")
   port map(
           rdaddress => wfifo_ra,
           wraddress => wfifo_wa,
           rdclock => dcc_clk,
           wrclock => clock,
           wren => wfifo_we,
           data => wfifo_din,
           q => wfifo_dout);
wfifo_din(34) <= lt_tsr(0);
wfifo_din(33) <= l_cmdo(0) or ld_addr;
wfifo_din(32) <= ld_addr;
wfifo_din(31 downto 0) <= l_dato when ld_addr = '0' else l_cmdo(0) & lt_tsr(9) & select_ram & x"000" & l_adro(16 downto 0);
wfifo_we <= (lt_tsr(0) xnor ld_addr) or (lt_tsr(0) and l_cmdo(0) and not lt_dxfrn);
select_ram <= l_adro(16) or l_adro(15); -- dccv4
i_wfifo_wa: gray port map(clk => clock, clrn => dcc_ready, ec => wfifo_we, q => wfifo_wa);
i_wfifo_ra: gray port map(clk => dcc_clk, clrn => dcc_ready, ec => high_word, q => wfifo_ra);
process(clock,dcc_ready)
begin
  if(dcc_ready = '0')then
    ld_addr <= '0';
  elsif(clock'event and clock = '1')then
    ld_addr <= not lt_tsr(0);
  end if;
end process;
-- wfifo_ra is a gray counter
-- synchronize wfifo_wa and generate read enable
process(dcc_clk,dcc_ready)
begin
  if(dcc_ready = '0')then
    high_word <= '0';
    dav <= '0';
    sync_wwa <= "0000";
    sync2_wwa <= "0000";
  elsif(dcc_clk'event and dcc_clk = '1')then
    sync_wwa <= wfifo_wa;
    sync2_wwa <= sync_wwa;
    if(high_word = '0')then
      if(sync2_wwa = wfifo_ra)then
        dav <= '0';
      else
        dav <= '1';
      end if;
    end if;
    if(sync2_wwa = wfifo_ra)then
      high_word <= '0';
    else
      high_word <= not high_word;
    end if;
  end if;
end process;
-- dcc_data
g_dd: for i in dcc_data'range generate
  dcc_data(i) <= dcc_data_o(i) when t_dcc_data(i) = '0' else 'Z';
end generate;
process(dcc_clk)
begin
  if(dcc_clk'event and dcc_clk = '1')then
    if(high_word = '1')then
      dcc_data_o <= wfifo_dout(31 downto 16);
    else
      dcc_data_o <= wfifo_dout(15 downto 0);
    end if;
    if(dav = '1')then
      dcc_ctl_i(2) <= wfifo_dout(34);
    end if; 
  end if;
end process;
-- dcc_ctl
-- pci3_cycle
process(dcc_clk,dcc_ready)
begin
  if(dcc_ready = '0')then
    pci_cycle <= '0';
    dcc_ctl_i(1 downto 0) <= "00";
    t_dcc_data <= (others => '1');
  elsif(dcc_clk'event and dcc_clk = '1')then
-- read address is NOP
-- data/address
    dcc_ctl_i(0) <= dav and wfifo_dout(34) and not wfifo_dout(32);
-- write/read
    dcc_ctl_i(1) <= dav and wfifo_dout(34) and wfifo_dout(33);
    for i in dcc_data'range loop
      t_dcc_data(i) <= not(dav and wfifo_dout(34) and wfifo_dout(33));
    end loop;
    if(dav = '1')then
      pci_cycle <= wfifo_dout(34);
    end if;
  end if;
end process;
-------------------------------------------
rfifo_weh <= is_high and not dcc_ctl_oq(0); 
process(dcc_clk,dcc_ready)
begin
  if(dcc_ready = '0')then
    rfifo_wel <= '0';
    dcc_ctl_oq <= "111111";
  elsif(dcc_clk'event and dcc_clk = '1')then
    dcc_ctl_oq <= dcc_ctl_o;
    rfifo_wel <= rfifo_weh and pci_cycle;
  end if;
end process;
process(dcc_clk)
begin
  if(dcc_clk'event and dcc_clk = '1')then
    is_high <= dcc_ctl_oq(0) or not is_high;
    dcc_data_i <= dcc_data;
  end if;
end process;
----------------------------------------------------------------------------------------------------------
-- instantiate read fifo's dpram
rfifo_h: lpm_ram_dp
--   generic map(LPM_WIDTH => 16)
   generic map(LPM_WIDTH => 16,LPM_WIDTHAD => 5,LPM_NUMWORDS => 32)
   port map(
           rdaddress => rfifo_ra,
           wraddress => rfifo_wa,
           rdclock => clock,
           wrclock => dcc_clk,
           rden => rfifo_re,
           wren => rfifo_weh,
           data => dcc_data_i,
           q => rfifo_dout(31 downto 16));
rfifo_l: lpm_ram_dp
--   generic map(LPM_WIDTH => 16)
   generic map(LPM_WIDTH => 16,LPM_WIDTHAD => 5,LPM_NUMWORDS => 32)
   port map(
           rdaddress => rfifo_ra,
           wraddress => rfifo_wa,
           rdclock => clock,
           wrclock => dcc_clk,
           rden => rfifo_re,
           wren => rfifo_wel,
           data => dcc_data_i,
           q => rfifo_dout(15 downto 0));
i_rfifo_ra: gray generic map(n=>5) port map(clk => clock, clrn => lt_tsr(0), ec => rfifo_re, q => rfifo_ra);
i_rfifo_wa: gray generic map(n=>5) port map(clk => dcc_clk, clrn => pci_cycle, ec => rfifo_wel, q => rfifo_wa);
rfifo_re <= not rfifo_empty and (not dcc_rd_rdy or not lt_dxfrn); 
rfifo_empty <= '1' when rfifo_ra = wr_ad_sync else '0';
process(clock,lt_tsr)
begin
  if(lt_tsr(0) = '0')then
    dcc_rd_rdy <= '0';
    wr_ad_syncp <= "00000";
    wr_ad_sync <= "00000";
    rd_addr <= "00000";
  elsif(clock'event and clock = '1')then
-- synchronize fifo write address
--    dcc_rd_rdy <= not rfifo_empty or (dcc_rd_rdy and lt_dxfrn);
    if(rfifo_empty = '0')then
      dcc_rd_rdy <= '1';
    elsif(lt_dxfrn = '0' and rd_addr /= "11111")then
      dcc_rd_rdy <= '0';
    end if;
    wr_ad_syncp <= rfifo_wa;
    wr_ad_sync <= wr_ad_syncp;
    if(ld_addr = '1' and l_cmdo(0) = '0')then
      rd_addr <= l_adro(6 downto 2);
    elsif(lt_dxfrn = '0' and rd_addr /= "11111")then
      rd_addr <= rd_addr + 1;
    end if;
  end if;
end process;
------------------------------------------------------------------------------------------            
dcc_ctl_i(3) <= not rst_dccn;
process(clock)
begin
  if(clock'event and clock = '1')then
-- pci_sp_i(3) is virtex2 done signal. Reset is delayed until done is high
    if(pci_sp_i(3) = '0')then
      rst_dccn <= '0';
    elsif(dcc_init(18) = '1')then
      rst_dccn <= '1';
    elsif(lt_tsr(1) = '1' and lt_dxfrn = '0')then
      if(l_adro(3 downto 2) = "10" and l_dato = x"5555aaaa" and l_cmdo(0) = '1')then
        rst_dccn <= '0';
      end if;
    end if;
  end if;
end process;
-- configuration interface
process(clock,rstn)
begin
  if(rstn = '0')then
    start <= '0';
    conf_data <= '0';
    conf_write <= '0';
    pci_sp_o(6 downto 5) <= "00";
    dcc_ready <= '0';
    dcc_init <= (others => '0');
  elsif(clock'event and clock = '1')then
    if(dcc_ready = '0' and pci_sp_i(3) = '1')then
       dcc_init <= dcc_init + 1;
    else
       dcc_init <= (others => '0');
    end if;
    if(rst_dccn = '0')then
      dcc_ready <= '0';
    end if;
    if(dcc_init(20) = '1')then
      dcc_ready <= '1';
    end if;
    start <= not lt_dxfrn and lt_tsr(1) and not(l_adro(3) and not l_adro(2));
    if(lt_tsr(1) = '1' and lt_dxfrn = '0')then
-- this is for starting reconfiguration
      pci_sp_o(5) <= l_adro(3);
      conf_data <= l_adro(2);
      conf_write <= l_cmdo(0);
      if(l_adro(2) = '0')then
        pci_sp_o(6) <= l_dato(20);
      end if;
    end if;
  end if;
end process;
-- read back
l_adi <= x"00" & conf_tmp1 when lt_tsr(1) = '1' else 
         rfifo_dout(7 downto 0) & rfifo_dout(15 downto 8) & rfifo_dout(23 downto 16) & rfifo_dout(31 downto 24) 
         when l_adro(17 downto 15) = "101" else rfifo_dout;
process(clock)
begin
  if(clock'event and clock = '1')then
--    ready1 <= (lt_tsr(0) and l_cmdo(0)) or (not en_count and lt_tsr(1));
-- modified 1-06-2007, pci_sp_i(3) is Xilinx done signal, this avoids PCI hang when Xilinx is down
    if(lt_tsr(1) = '0')then
      en_count_q <= en_count;
    end if;
    ready1 <= (lt_tsr(0) and (l_cmdo(0) or not pci_sp_i(3))) or (not en_count_q and lt_tsr(1));
    retry <= retry(0) & dcc_ctl_oq(1);
    discon <= lt_framen or (lt_tsr(0) and (l_cmdo(0) or (retry(1) xnor retry(0))));
-- single read has different lt_discn timing
    bar1_read <= not l_cmdo(0) and not en_count_q and  lt_tsr(1);
  end if;
end process;
lt_discn <= lt_dxfrn when bar1_read = '1' else discon;            
lt_abortn <= '1'; 
--
pci_sp_o(8 downto 7) <= "00";
lt_rdyn <= not(ready1 or (dcc_rd_rdy and (not rfifo_empty or lt_dxfrn)));
-- all bytes enabled
l_cbeni <= "1111";
-------------------------------------configuration control cpld interface
conf_clk <= pci_sp_i(1);
process(clock)
begin
  if(clock'event and clock = '1')then
    if(conf_tgl_sync(2) /= conf_tgl_sync(1))then
	   shift_sync <= shift_sync(1 downto 0) & shift_flag_p;
	   input_sel <= shift_sync(2) and not input_sel;
	 end if;
    if(conf_tgl_sync(2) /= conf_tgl_sync(1) and count /= 0)then
	   out_shift <= '1';
	 else
	   out_shift <= '0';
	 end if;
	 if(shift_sync(2) = '1' and conf_tgl_sync(2) /= conf_tgl_sync(1))then
	   conf_tmp1(23 downto 17) <= '0' & conf_tmp1(21 downto 16);
       conf_tmp1(16) <= ver_din_q;
       ver_din_q <= ver_din;
	   conf_tmp1(13 downto 0) <= conf_tmp1(12 downto 0) & conf_din;
	 end if;
	 if(lt_tsr(1) = '1' and lt_dxfrn = '0')then
	   conf_tmp0 <= l_dato(19 downto 0);
	 elsif(out_shift = '1')then
	   conf_tmp0 <= conf_tmp0(18 downto 0) & '0';
	 end if;
	 if(out_shift = '1')then
	   if(count(0) = '0')then
	     shift_data_out(0) <= conf_dout;
	   else
	     shift_data_out(1) <= conf_dout;
		end if;
	 end if;
  end if;
end process;
conf_tmp1(15 downto 14) <= "00";
conf_dout <= conf_tmp0(7) when conf_data = '1' else conf_tmp0(19);
conf_din <= shift_data_in(0) when input_sel = '0' else shift_data_in(1);
ver_din <= shift_ver_in(0) when input_sel = '0' else shift_ver_in(1);
process(clock,rstn)
begin
  if(rstn = '0')then
    start_sync <= '0';
    en_count <= '0';
    en_count_dl <= '0';
    count <= "00000";
    conf_tgl_sync <= "000";
  elsif(clock'event and clock = '1')then
    start_sync <= start or (start_sync and not(conf_tgl_sync(2) xor conf_tgl_sync(1)));
    if(start_sync = '1' and conf_tgl_sync(2) /= conf_tgl_sync(1))then
	   en_count <= '1';
	 elsif(count = 30)then
	   en_count <= '0';
	 end if;
    if(conf_tgl_sync(2) /= conf_tgl_sync(1))then
      en_count_dl <= en_count;
    end if;
	 if(en_count_dl = '0')then
	   count <= "00000";
    elsif(ec_count = '1')then
	   count <= count + 1;
	 end if;
	 conf_tgl_sync <= conf_tgl_sync(1 downto 0) & conf_tgl;	 
	 if(count(0) = '0')then
	   case count(4 downto 1) is
		  when x"1" | x"2" | x"3" | x"4" | x"5" | x"6" | x"7" | x"8" | x"9" | x"a" => load_addr(0) <= en_count and not conf_data;
		  when others => load_addr(0) <= '0';
		end case;
	   case count(4 downto 1) is
		  when x"4" | x"5" | x"6" => write_flash(0) <= conf_write and conf_data;
		  when others => write_flash(0) <= '0';
		end case;
	   case count(4 downto 1) is
		  when x"1" | x"2" | x"3" | x"4" | x"6" => shift_flag(0) <= conf_write and conf_data;
		  when x"9" | x"a" | x"b" | x"c" | x"d" | x"e" | x"f" => shift_flag(0) <= '1';
		  when others => shift_flag(0) <= '0';
		end case;
	 else
	   if(count(4 downto 1) = 11)then
	     read_flash <= '1';
	   else
	     read_flash <= '0';
	   end if;
	   case count(4 downto 1) is
		  when x"0" | x"1" | x"2" | x"3" | x"4" | x"5" | x"6" | x"7" | x"8" | x"9" => load_addr(1) <= not conf_data;
		  when others => load_addr(1) <= '0';
		end case;
	   case count(4 downto 1) is
		  when x"4" | x"5" => write_flash(1) <= conf_write and conf_data;
		  when others => write_flash(1) <= '0';
		end case;
	   case count(4 downto 1) is
		  when x"0" | x"1" | x"2" | x"3" | x"5" | x"6" => shift_flag(1) <= conf_write and conf_data;
		  when x"8" | x"9" | x"a" | x"b" | x"c" | x"d" | x"e" => shift_flag(1) <= '1';
		  when others => shift_flag(1) <= '0';
		end case;
	 end if;	 
  end if;
end process;
ec_count <= en_count_dl and (conf_tgl_sync(2) xor conf_tgl_sync(1));
process(conf_clk)
begin
  if(conf_clk'event and conf_clk = '1')then
    en_count_sync <= en_count_sync(1 downto 0) & en_count; 
	 conf_tgl <= not conf_tgl;
	 conf_sel <= (en_count_sync(1) or not en_count_sync(0)) and not conf_sel;
	 if(en_count_sync(1) = '0')then
	   pci_sp_o(4 downto 0) <= "00000";
	 elsif(conf_sel = '0')then
	   pci_sp_o(0) <= load_addr(0);
	   pci_sp_o(1) <= '0';
	   pci_sp_o(2) <= write_flash(0);
	   pci_sp_o(3) <= shift_flag(0);
	   pci_sp_o(4) <= shift_data_out(0);
	 else
	   pci_sp_o(0) <= load_addr(1);
	   pci_sp_o(1) <= read_flash;
	   pci_sp_o(2) <= write_flash(1);
	   pci_sp_o(3) <= shift_flag(1);
	   pci_sp_o(4) <= shift_data_out(1);
	 end if;
	 if(en_count_sync(1) = '0')then
	   shift_flag_p <= '0';
	 elsif(conf_sel = '0')then
      shift_flag_p <= shift_flag(0);
	 else
      shift_flag_p <= shift_flag(1);
	 end if;
  end if;
end process;
process(conf_clk)
begin
  if(conf_clk'event and conf_clk = '0')then
	 if(conf_sel = '0')then
	   shift_data_in(0) <= pci_sp_i(0);
	   shift_ver_in(0) <= pci_sp_i(2);
	 else
	   shift_data_in(1) <= pci_sp_i(0);
	   shift_ver_in(1) <= pci_sp_i(2);
	 end if;
  end if;
end process;
END my_arch;

